
// syncal.cc
// 3/11/99
//
// This is the calibration software for the AOE synthesizer.  Calibration
// is used to compensate for high-frequency rolloff of the synthesizer's
// amplifiers to provide a flat signal output over the entire operating
// bandwidth.


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "syncal.h"
#include "serial.h"
#include "kbd.h"
#include "support.h"


int * in = NULL;		// the array of ints for data exchange
				// with microcontroller


// attempt to communicate with microcontroller.  if possible, prompt the
// user with a choice of disabling calibration, calibrating the device,
// or exiting.  perform the appropriate action and exit.
int main(int argc, char ** argv)
{
	int choice;
	in = new int[MAX_BYTES];

	system("clear");
	printf("\nSYNTHESIZER CALIBRATION\n\n");
	init_key();		// initialize keyboard
	init_serial();		// and serial port

	micro_status();		// attempt to establish communication
				// with microcontroller

	choice = menu();	// user picks an option
	if (choice == 'b')	// disable calibration, then calibrate
	{
		cal_disable();
		calibrate();
	}
	else if (choice == 'a') cal_disable();	// disable calibration

	printf("\n");
	shutdown_serial();	// clean up
	shutdown_key();
	delete in;
	printf("\n\n");
}


// display program options, await user input, return the key that was pressed
int menu()
{
	int key=0;
	printf("\nOPTIONS\na) disable calibration\nb) calibrate\nq) exit\n");
	while (key != 'a' && key != 'b' && key != 'q') key = getkey();

	return key;
}


// display calibration instructions
void instructions()
{
	system("clear");
	printf("\n\nCALIBRATION INSTRUCTIONS\nThis program sets the frequency"
		" of the synthesizer, and asks you to\nlevel the output using"
		" the variable attenuator.  At the first (highest)\n"
		"frequency, mark the signal amplitude.  Then, at subsequent"
		" frequencies,\nset the attenuation such that the signal"
		" level is the same as the level\nat the highest"
		" frequency.\n\n");
}


// disable calibration.  send the reset code to the microcontroller,
// wait for echo of command.
void cal_disable()
{
	send_cmd(cal_reset,0,in);
	confirm(cal_reset);
}


// calibrate the synthesizer.  first, acquire the calibration data, then
// process it, and send it to the microcontroller.  wait for an echo
// of the calibration data command from the microcontroller.
// if the calibration data is not valid, don't send anything; instead,
// terminate the program.
void calibrate()
{
	double * data = NULL;
	int go;

	instructions();
	data = acquire_cal_data();	
	go = process_data(data);
	if (go)
	{
		send_cmd(cal_data,NUMP-1,in);
		confirm(cal_data);
	}
	else
	{
		printf("\nbad calibration data\n");
		terminate();
	}
}


// attempt to establish communication with microcontroller.  if micro does
// not echo the status command, timeout, and terminate the program. 
void micro_status()
{
	long timeout = 10000;

	send_cmd(alive_cmd,0,in);
	while (--timeout)
	{
		if (getserial())
		{
			if (in[0] == alive_cmd) break;
		}
	}
	if (! timeout)
	{
	    printf("\nmicrocontroller not responding.  make sure synthesizer\n"
		"is turned on, and serial port is connected.\n");
	    terminate();
	}
}


// acquire the calibration data with the user's assistance.  FREQ_SET
// is an array containing the frequencies at which to acquire cal data.
// loop on this array, setting the frequency, waiting for a keystroke, and
// then reading the attenuator setting.
double * acquire_cal_data()
{
	double * data = new double[NUMP];
	int i;
	
	for (i=0; i < NUMP; i++)
	{
		set_freq(FREQ_SET[i]);
		printf("frequency set to %.1f :  press a key"
			" when ready\n",FREQ_SET[i]);
		while (! getkey());	
		data[i] = read_atten();
	}

	return data;
}


// set the synthesizer frequency (CHAN determines which channel is affected.
// since both channels use the same calibration data, it doesn't really matter. 
// reset the attenuator for this channel to zero.
void set_freq(double val)
{
	double att_setting = 0.0;

	if (CHAN) in[0] = 1;		// first byte holds mode info 
	else in[0] = 0;
	bin_to_decimal(val,&in[1],9);	// convert the frequency to decimal
	send_cmd(send_setting,10,in);	// change the frequency
	confirm(send_setting);		// wait for confirmation

	if (CHAN) in[0] = 3;		// now reset the attenuator to zero
	else in[0] = 2;
	bin_to_decimal(att_setting,&in[1],3);
	send_cmd(send_setting,4,in);
	confirm(send_setting);
}


// read the attenuation setting.  convert the returned decimal value into
// the floating point attenuation, and return this value.
double read_atten()
{
	if (CHAN) in[0] = 3;	// the argument is the mode being polled
	else in[0] = 2;

	send_cmd(get_setting,1,in);	// send the command
	confirm(get_setting);		// wait for the reply

	return (decimal_to_bin(&in[3],3));	// convert the reply and exit
}


// a code is passed to this function.  loop until the microcontroller
// passes this command code to the PC.  Display an appropriate message
// when received.  Exit if a timeout condition occurs or if the user 
// hits the 'q' key.
void confirm(int code)
{
	long timeout = 10000;

	while (--timeout)
	{
		if (getkey() == 'q') terminate();
		if (getserial())
		{
			if (in[0] == code) break;
		}	
	}
	
	if (! timeout) terminate();

	if (in[0] == cal_data) printf("\ncalibration data has been set\n\n");
	if (in[0] == cal_reset) printf("\ncalibration disabled\n\n");
} 


// a premature termination of this program.
void terminate()
{
	printf("\nterminating calibration\n\n");
	shutdown_serial();
	shutdown_key();
	delete in;
	printf("\n\n");
	exit(0);
}


// Convert the attenuation settings entered by the user into the calibration
// data.  Define the last attenuation returned (for the lowest frequency) 
// as X.  Then, for each higher frequency, (X - attenuation) is the calibration
// point.  This calibration point is the number of dB down the synthesizer
// output is at this frequency.  In practice, the calibration values will
// range from 0.0 dB to 5.0 dB.  These values must be converted into integer
// values (for use by the microcontroller), so they are multiplied by 10
// before they are converted to integers.  The calibration points are 
// stored in the "in" integer array for passing through the serial interface. 
// the calibration points are returned in order of increasing frequency.
// if any of the calibration points are negative, the data entered by
// the user is invalid.  this type of data will not be passed; instead,
// the program will terminate.
int process_data(double * data)
{
	int success = 1;
	int i,j=0;
	double * processed = new double[NUMP-1];

	for (i=0; i < (NUMP-1); i++)
		data[i] = data[NUMP-1] - data[i];   // X = NUMP - 1 (see above)
	for (i = NUMP-2; i >= 0; i--)
		processed[j++] = data[i];	// reverse the order

	for (i=0; i < NUMP-1; i++)
	{
		in[i] = (int) (processed[i] * 10.0 + 0.5);
		if (in[i] < 0) success = 0;
	}
	
	delete data;
	delete processed;

	return success;
}



// end of syncal.cc




